import os

PATH = os.path.dirname(os.path.realpath(__file__))
TEST_REQUIREMENTS_PATH = os.path.join(PATH, "fixtures/requirements-initial.txt")


def get_environment_requirements_list_monkey(*args, **kwargs):
    with open(TEST_REQUIREMENTS_PATH) as f:
        content = f.readlines()

    return content



import doctest
import sys
import unittest


def test_suite():
    suite = unittest.TestLoader().discover("tests")
    suite.addTest(doctest.DocFileSuite("README.rst"))
    return suite


if __name__ == "__main__":
    result = unittest.TextTestRunner(verbosity=2).run(test_suite())
    sys.exit(not result.wasSuccessful())



#!/usr/bin/env python
import asyncio
import sys
import unittest
from argparse import ArgumentTypeError
from io import StringIO
from unittest.mock import patch

from tests.utils import TEST_REQUIREMENTS_PATH, get_environment_requirements_list_monkey
from updatable.console import _argument_parser, _list_package_updates, _list_updates, _str_to_bool, _updatable


class Capture(list):
    def __enter__(self):
        self._stdout = sys.stdout
        sys.stdout = self._stringio = StringIO()
        return self

    def __exit__(self, *args):
        self.extend(self._stringio.getvalue().splitlines())
        del self._stringio  # free up memory
        sys.stdout = self._stdout


class TestStrToBool(unittest.TestCase):
    def test_str_to_bool_true(self):
        self.assertTrue(_str_to_bool(True))
        self.assertTrue(_str_to_bool("t"))
        self.assertTrue(_str_to_bool("y"))
        self.assertTrue(_str_to_bool("True"))
        self.assertTrue(_str_to_bool("YES"))
        self.assertTrue(_str_to_bool("1"))

    def test_str_to_bool_false(self):
        self.assertFalse(_str_to_bool(False))
        self.assertFalse(_str_to_bool("f"))
        self.assertFalse(_str_to_bool("n"))
        self.assertFalse(_str_to_bool("False"))
        self.assertFalse(_str_to_bool("NO"))
        self.assertFalse(_str_to_bool("0"))

    def test_str_to_bool_exception(self):
        with self.assertRaises(ArgumentTypeError):
            _str_to_bool("")

        with self.assertRaises(ArgumentTypeError):
            _str_to_bool("falsy")

        with self.assertRaises(ArgumentTypeError):
            _str_to_bool("eye")


class TestListUpdates(unittest.TestCase):
    def test_with_empty_list(self):
        with Capture() as output:
            _list_updates("Test", [], "MIT")

        self.assertListEqual(output, [])

    def test_with_updates_in_list(self):
        with Capture() as output:
            _list_updates(
                "Test",
                [
                    {"version": "1.0.0", "upload_time": "date 1"},
                    {"version": "2.0.0", "upload_time": "date 2"},
                ],
                "MIT",
            )

        self.assertListEqual(
            output,
            [
                "  Test:",
                "  -- 1.0.0 on date 1 - License: MIT",
                "  -- 2.0.0 on date 2 - License: MIT",
            ],
        )


class TestListPackageUpdates(unittest.TestCase):
    async def _mock_get_package_update_list(*args, **kwargs):
        # No updates, no prereeases, no non semantic version
        if args[1] == "package1":
            return {
                "newer_releases": 0,
                "pre_releases": 0,
                "major_updates": [],
                "minor_updates": [],
                "patch_updates": [],
                "pre_release_updates": [],
                "non_semantic_versions": [],
                "current_release_license": "MIT",
            }

        # Updates, no prereeases, no non semantic version
        if args[1] == "package2":
            return {
                "newer_releases": 5,
                "pre_releases": 0,
                "major_updates": [
                    {"version": "2.0.0", "upload_time": "date 3"},
                    {"version": "3.0.0", "upload_time": "date 5"},
                ],
                "minor_updates": [
                    {"version": "1.5.0", "upload_time": "date 2"},
                    {"version": "2.5.0", "upload_time": "date 4"},
                ],
                "patch_updates": [
                    {"version": "1.5.5", "upload_time": "date 5"},
                ],
                "pre_release_updates": [],
                "non_semantic_versions": [],
                "current_release_license": "MIT",
            }

        # Updates, no prereeases, non semantic version
        if args[1] == "package3":
            return {
                "newer_releases": 1,
                "pre_releases": 0,
                "major_updates": [],
                "minor_updates": [],
                "patch_updates": [
                    {"version": "1.5.5", "upload_time": "date 5"},
                ],
                "pre_release_updates": [],
                "non_semantic_versions": [
                    {"version": "test1.5.5.3.2.3.23", "upload_time": "date 6"},
                ],
                "current_release_license": "MIT",
            }

        # Updates, prereeases, non semantic version
        if args[1] == "package4":
            return {
                "newer_releases": 1,
                "pre_releases": 1,
                "major_updates": [],
                "minor_updates": [],
                "patch_updates": [
                    {"version": "1.5.5", "upload_time": "date 5"},
                ],
                "pre_release_updates": [
                    {"version": "alfa-1.5.5", "upload_time": "date 7"},
                ],
                "non_semantic_versions": [
                    {"version": "test1.5.5.3.2.3.23", "upload_time": "date 6"},
                ],
                "current_release_license": "MIT",
            }

        # No updates, prereeases, non semantic version
        if args[1] == "package5":
            return {
                "newer_releases": 0,
                "pre_releases": 1,
                "major_updates": [],
                "minor_updates": [],
                "patch_updates": [],
                "pre_release_updates": [
                    {"version": "alfa-1.5.5", "upload_time": "date 7"},
                ],
                "non_semantic_versions": [
                    {"version": "test1.5.5.3.2.3.23", "upload_time": "date 6"},
                ],
                "current_release_license": "MIT",
            }

        # Pre releases only
        if args[1] == "package6":
            return {
                "newer_releases": 0,
                "pre_releases": 1,
                "major_updates": [],
                "minor_updates": [],
                "patch_updates": [],
                "pre_release_updates": [
                    {"version": "alfa-1.5.5", "upload_time": "date 7"},
                ],
                "non_semantic_versions": [],
                "current_release_license": "MIT",
            }

        # Non semantic version only
        if args[1] == "package7":
            return {
                "newer_releases": 0,
                "pre_releases": 0,
                "major_updates": [],
                "minor_updates": [],
                "patch_updates": [],
                "pre_release_updates": [],
                "non_semantic_versions": [
                    {"version": "test1.5.5.3.2.3.23", "upload_time": "date 6"},
                ],
                "current_release_license": "MIT",
            }

    def _mock_argument_parser(*args, **kwargs):
        class MockResult:
            file = get_environment_requirements_list_monkey()
            pre_releases = False

        class ArgumentParserMock:
            def parse_args(*args, **kwargs):
                return MockResult()

        return ArgumentParserMock()

    def test_with_no_available_updates(self):
        with patch(
            "updatable.utils.get_package_update_list",
            side_effect=self._mock_get_package_update_list,
        ):
            with Capture() as output:
                asyncio.run(_list_package_updates("package1", "1.0.0", False))
            self.assertListEqual(output, [])

            with Capture() as output:
                asyncio.run(_list_package_updates("package1", "1.0.0", True))
            self.assertListEqual(output, [])

    def test_with_updates_and_no_prereleases(self):
        with patch(
            "updatable.utils.get_package_update_list",
            side_effect=self._mock_get_package_update_list,
        ):
            with Capture() as output:
                asyncio.run(_list_package_updates("package2", "1.0.0", False))
            self.assertListEqual(
                output,
                [
                    "package2 (1.0.0) - License: MIT",
                    "  Major releases:",
                    "  -- 2.0.0 on date 3 - License: MIT",
                    "  -- 3.0.0 on date 5 - License: MIT",
                    "  Minor releases:",
                    "  -- 1.5.0 on date 2 - License: MIT",
                    "  -- 2.5.0 on date 4 - License: MIT",
                    "  Patch releases:",
                    "  -- 1.5.5 on date 5 - License: MIT",
                    "___",
                ],
            )

            with Capture() as output:
                asyncio.run(_list_package_updates("package2", "1.0.0", True))
            self.assertListEqual(
                output,
                [
                    "package2 (1.0.0) - License: MIT",
                    "  Major releases:",
                    "  -- 2.0.0 on date 3 - License: MIT",
                    "  -- 3.0.0 on date 5 - License: MIT",
                    "  Minor releases:",
                    "  -- 1.5.0 on date 2 - License: MIT",
                    "  -- 2.5.0 on date 4 - License: MIT",
                    "  Patch releases:",
                    "  -- 1.5.5 on date 5 - License: MIT",
                    "___",
                ],
            )

    def test_with_updates_and_no_prereleases_and_non_semantic_versions(self):
        with patch(
            "updatable.utils.get_package_update_list",
            side_effect=self._mock_get_package_update_list,
        ):
            with Capture() as output:
                asyncio.run(_list_package_updates("package3", "1.0.0", False))
            self.assertListEqual(
                output,
                [
                    "package3 (1.0.0) - License: MIT",
                    "  Patch releases:",
                    "  -- 1.5.5 on date 5 - License: MIT",
                    "  Unknown releases:",
                    "  -- test1.5.5.3.2.3.23 on date 6 - License: MIT",
                    "___",
                ],
            )

            with Capture() as output:
                asyncio.run(_list_package_updates("package3", "1.0.0", True))
            self.assertListEqual(
                output,
                [
                    "package3 (1.0.0) - License: MIT",
                    "  Patch releases:",
                    "  -- 1.5.5 on date 5 - License: MIT",
                    "  Unknown releases:",
                    "  -- test1.5.5.3.2.3.23 on date 6 - License: MIT",
                    "___",
                ],
            )

    def test_with_updates_and_prereleases_and_non_semantic_versions(self):
        with patch(
            "updatable.utils.get_package_update_list",
            side_effect=self._mock_get_package_update_list,
        ):
            with Capture() as output:
                asyncio.run(_list_package_updates("package4", "1.0.0", False))
            self.assertListEqual(
                output,
                [
                    "package4 (1.0.0) - License: MIT",
                    "  Patch releases:",
                    "  -- 1.5.5 on date 5 - License: MIT",
                    "  Unknown releases:",
                    "  -- test1.5.5.3.2.3.23 on date 6 - License: MIT",
                    "___",
                ],
            )

            with Capture() as output:
                asyncio.run(_list_package_updates("package4", "1.0.0", True))
            self.assertListEqual(
                output,
                [
                    "package4 (1.0.0) - License: MIT",
                    "  Patch releases:",
                    "  -- 1.5.5 on date 5 - License: MIT",
                    "  Unknown releases:",
                    "  -- test1.5.5.3.2.3.23 on date 6 - License: MIT",
                    "  Pre releases:",
                    "  -- alfa-1.5.5 on date 7 - License: MIT",
                    "___",
                ],
            )

    def test_with_prereleases_and_non_semantic_versions(self):
        with patch(
            "updatable.utils.get_package_update_list",
            side_effect=self._mock_get_package_update_list,
        ):
            with Capture() as output:
                asyncio.run(_list_package_updates("package5", "1.0.0", False))
            self.assertListEqual(output, [])

            with Capture() as output:
                asyncio.run(_list_package_updates("package5", "1.0.0", True))
            self.assertListEqual(
                output,
                [
                    "package5 (1.0.0) - License: MIT",
                    "  Pre releases:",
                    "  -- alfa-1.5.5 on date 7 - License: MIT",
                    "___",
                ],
            )

    def test_with_prereleases(self):
        with patch(
            "updatable.utils.get_package_update_list",
            side_effect=self._mock_get_package_update_list,
        ):
            with Capture() as output:
                asyncio.run(_list_package_updates("package6", "1.0.0", False))
            self.assertListEqual(output, [])

            with Capture() as output:
                asyncio.run(_list_package_updates("package6", "1.0.0", True))
            self.assertListEqual(
                output,
                [
                    "package6 (1.0.0) - License: MIT",
                    "  Pre releases:",
                    "  -- alfa-1.5.5 on date 7 - License: MIT",
                    "___",
                ],
            )

    def test_with_non_semantic_versions(self):
        with patch(
            "updatable.utils.get_package_update_list",
            side_effect=self._mock_get_package_update_list,
        ):
            with Capture() as output:
                asyncio.run(_list_package_updates("package7", "1.0.0", False))
            self.assertListEqual(output, [])

            with Capture() as output:
                asyncio.run(_list_package_updates("package7", "1.0.0", True))
            self.assertListEqual(output, [])

    def test_updatable_call(self):
        with patch("updatable.console._argument_parser", side_effect=self._mock_argument_parser):
            with patch(
                "updatable.utils.get_package_update_list",
                side_effect=self._mock_get_package_update_list,
            ):
                with Capture() as output:
                    asyncio.run(_updatable())

                self.assertListEqual(
                    output,
                    [
                        "package2 (1.0) - License: MIT",
                        "  Major releases:",
                        "  -- 2.0.0 on date 3 - License: MIT",
                        "  -- 3.0.0 on date 5 - License: MIT",
                        "  Minor releases:",
                        "  -- 1.5.0 on date 2 - License: MIT",
                        "  -- 2.5.0 on date 4 - License: MIT",
                        "  Patch releases:",
                        "  -- 1.5.5 on date 5 - License: MIT",
                        "___",
                        "package3 (2) - License: MIT",
                        "  Patch releases:",
                        "  -- 1.5.5 on date 5 - License: MIT",
                        "  Unknown releases:",
                        "  -- test1.5.5.3.2.3.23 on date 6 - License: MIT",
                        "___",
                        "package4 (2.4) - License: MIT",
                        "  Patch releases:",
                        "  -- 1.5.5 on date 5 - License: MIT",
                        "  Unknown releases:",
                        "  -- test1.5.5.3.2.3.23 on date 6 - License: MIT",
                        "___",
                    ],
                )


class TestArgumentParser(unittest.TestCase):
    def setUp(self):
        self.parser = _argument_parser()

    def test_argument_parser_pre_release(self):
        # Long param
        parsed = self.parser.parse_args(["--pre-release", "True"])
        self.assertEqual(parsed.pre_releases, True)

        parsed = self.parser.parse_args(["--pre-release", "Yes"])
        self.assertEqual(parsed.pre_releases, True)

        parsed = self.parser.parse_args(["--pre-release", "T"])
        self.assertEqual(parsed.pre_releases, True)

        parsed = self.parser.parse_args(["--pre-release", "t"])
        self.assertEqual(parsed.pre_releases, True)

        parsed = self.parser.parse_args(["--pre-release", "1"])
        self.assertEqual(parsed.pre_releases, True)

        parsed = self.parser.parse_args(["--pre-release", "TrUe"])
        self.assertEqual(parsed.pre_releases, True)

        parsed = self.parser.parse_args(["--pre-release", "f"])
        self.assertEqual(parsed.pre_releases, False)

        parsed = self.parser.parse_args(["--pre-release", "n"])
        self.assertEqual(parsed.pre_releases, False)

        parsed = self.parser.parse_args(["--pre-release", "0"])
        self.assertEqual(parsed.pre_releases, False)

        parsed = self.parser.parse_args(["--pre-release", "FaLse"])
        self.assertEqual(parsed.pre_releases, False)

        with self.assertRaises(SystemExit):
            self.parser.parse_args(["--pre-release", "Invalid"])

        # Short param
        parsed = self.parser.parse_args(["-pr", "True"])
        self.assertEqual(parsed.pre_releases, True)

        parsed = self.parser.parse_args(["-pr", "Yes"])
        self.assertEqual(parsed.pre_releases, True)

        parsed = self.parser.parse_args(["-pr", "T"])
        self.assertEqual(parsed.pre_releases, True)

        parsed = self.parser.parse_args(["-pr", "t"])
        self.assertEqual(parsed.pre_releases, True)

        parsed = self.parser.parse_args(["-pr", "1"])
        self.assertEqual(parsed.pre_releases, True)

        parsed = self.parser.parse_args(["-pr", "TrUe"])
        self.assertEqual(parsed.pre_releases, True)

        parsed = self.parser.parse_args(["-pr", "f"])
        self.assertEqual(parsed.pre_releases, False)

        parsed = self.parser.parse_args(["-pr", "n"])
        self.assertEqual(parsed.pre_releases, False)

        parsed = self.parser.parse_args(["-pr", "0"])
        self.assertEqual(parsed.pre_releases, False)

        parsed = self.parser.parse_args(["-pr", "FaLse"])
        self.assertEqual(parsed.pre_releases, False)

        with self.assertRaises(SystemExit):
            self.parser.parse_args(["-pr", "Invalid"])

    def test_invalid_argument(self):
        with self.assertRaises(SystemExit):
            self.parser.parse_args(["--invalid", "Value"])

    def test_argument_parser_pre_file(self):
        # Long param
        parsed = self.parser.parse_args(["--file", TEST_REQUIREMENTS_PATH])

        self.assertEqual(
            list(parsed.file),
            [
                "package1==0.1\n",
                "package2==1.0\n",
                "package3==2\n",
                "package4==2.4\n",
                "package5==3.0.0\n",
            ],
        )

        with self.assertRaises(SystemExit):
            self.parser.parse_args(["--file", "Invalid"])

        # Short param
        parsed = self.parser.parse_args(["-f", TEST_REQUIREMENTS_PATH])

        self.assertEqual(
            list(parsed.file),
            [
                "package1==0.1\n",
                "package2==1.0\n",
                "package3==2\n",
                "package4==2.4\n",
                "package5==3.0.0\n",
            ],
        )

        with self.assertRaises(SystemExit):
            self.parser.parse_args(["-f", "Invalid"])


if __name__ == "__main__":
    unittest.main()



import argparse
import asyncio
import datetime

from updatable import utils as updatable_utils


def _str_to_bool(value):
    """
    Converts a string into a bool

    :param pacvaluekage_name: string
    """
    if isinstance(value, bool):
        return value
    if value.lower() in ("yes", "true", "t", "y", "1"):
        return True
    elif value.lower() in ("no", "false", "f", "n", "0"):
        return False
    else:
        raise argparse.ArgumentTypeError("Boolean value expected!")


async def _list_package_updates(package_name, version, show_pre_releases=False):
    """
    Function used to list all package updates in console

    :param package_name: string
    :param version: string
    :param show_pre_releases bool
    """
    updates = await updatable_utils.get_package_update_list(package_name, version)
    has_displayed_updates = updates["newer_releases"] or (show_pre_releases and updates["pre_releases"])
    current_release_license = updates["current_release_license"]

    if has_displayed_updates:
        print(f"{package_name} ({version}) - License: {current_release_license}")

    if updates["newer_releases"]:
        _list_updates("Major releases", updates["major_updates"], current_release_license)
        _list_updates("Minor releases", updates["minor_updates"], current_release_license)
        _list_updates("Patch releases", updates["patch_updates"], current_release_license)
        _list_updates(
            "Unknown releases",
            updates["non_semantic_versions"],
            current_release_license,
        )
        has_displayed_updates = True

    if show_pre_releases and updates["pre_releases"]:
        _list_updates("Pre releases", updates["pre_release_updates"], current_release_license)
        has_displayed_updates = True

    if has_displayed_updates:
        print("___")


def _list_updates(update_type, update_list, current_release_license):
    """
    Function used to list package updates by update type in console

    :param update_type: string
    :param update_list: list
    :param current_release_license: string
    """
    if len(update_list):
        print(f"  {update_type}:")
        for update_item in update_list:
            print(
                f"  -- {update_item['version']} on {update_item['upload_time']} - License: {current_release_license}",
            )


def _argument_parser():
    """
    Configure arguments for console
    """
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "-f",
        "--file",
        nargs="?",
        type=argparse.FileType(),
        default=None,
        help="Requirements file",
    )
    parser.add_argument(
        "-pr",
        "--pre-releases",
        nargs="?",
        type=_str_to_bool,
        default=False,
        help="Show pre-releases",
    )

    return parser


async def _updatable():
    """
    Function used to output packages update information in the console
    """
    args = _argument_parser().parse_args()

    # Get list of packages
    if args.file:
        packages = updatable_utils.parse_requirements_list(args.file)
    else:
        packages = updatable_utils.get_parsed_environment_package_list()

    # Output updates
    tasks = []
    for package in packages:
        tasks.append(
            asyncio.create_task(_list_package_updates(package["package"], package["version"], args.pre_releases)),
        )

    for task in tasks:
        await task


def main():  # pragma: no cover
    t0 = datetime.datetime.now()
    asyncio.run(_updatable())
    dt = datetime.datetime.now() - t0
    print(f"Done in {dt.total_seconds():.2f} sec.")



import re
import sys
from datetime import datetime
from subprocess import check_output

import httpx
import semantic_version
from packaging.version import parse

__all__ = [
    "is_major_update",
    "is_minor_update",
    "is_patch_update",
    "sorted_versions",
    "get_categorized_package_data",
    "get_parsed_environment_package_list",
    "get_environment_requirements_list",
    "parse_requirements_list",
    "get_pypi_package_data",
    "get_package_update_list",
]


def is_major_update(release, package):
    """
    Checks if the release is a major update compared to the package

    :param release: semantic_version.Version
    :param package: semantic_version.Version
    :return: bool
    """
    return release in semantic_version.SimpleSpec(f">={package.next_major()}")


def is_minor_update(release, package):
    """
    Checks if the release is a minor update compared to the package

    :param release: semantic_version.Version
    :param package: semantic_version.Version
    :return: bool
    """
    return release in semantic_version.SimpleSpec(f">={package.next_minor()},<{package.next_major()}")


def is_patch_update(release, package):
    """
    Checks if the release is a patch update compared to the package

    :param release: semantic_version.Version
    :param package: semantic_version.Version
    :return: bool
    """
    return release in semantic_version.SimpleSpec(f">={package.next_patch()},<{package.next_minor()}")


def sorted_versions(versions):
    """
    Returns the list of Versions in ascending order

    :param versions: semantic_version.Version[]
    :return: semantic_version.Version[]
    """
    return sorted(
        versions,
        key=lambda x: semantic_version.Version.coerce(x["version"]),
        reverse=True,
    )


def get_categorized_package_data(package_data, package_version):
    """
    Returns all Versions grouped by type compared to the current package version

    :param package_data: dict
    :param package_version: semantic_version.Version
    :return: {
        major_updates: semantic_version.Version[]
        minor_updates: semantic_version.Version[]
        patch_updates: semantic_version.Version[]
        pre_release_updates: semantic_version.Version[]
        non_semantic_versions: semantic_version.Version[]
    }
    """
    major_updates = []
    minor_updates = []
    patch_updates = []
    pre_release_updates = []
    non_semantic_versions = []

    for release, info in package_data["releases"].items():
        upload_time = None
        if info:
            upload_time = datetime.strptime(info[0]["upload_time"], "%Y-%m-%dT%H:%M:%S")

        try:
            parsed_release = parse(release)

            # Get semantic version of package
            release_version = semantic_version.Version.coerce(release)

            if not parsed_release.is_prerelease:
                # Place package in the appropriate semantic visioning list
                if is_major_update(release_version, package_version):
                    major_updates.append(
                        {
                            "version": release,
                            "upload_time": upload_time,
                        },
                    )
                elif is_minor_update(release_version, package_version):
                    minor_updates.append(
                        {
                            "version": release,
                            "upload_time": upload_time,
                        },
                    )
                elif is_patch_update(release_version, package_version):
                    patch_updates.append(
                        {
                            "version": release,
                            "upload_time": upload_time,
                        },
                    )
            else:
                pre_release_updates.append({"version": release, "upload_time": upload_time})
        except ValueError:
            # Keep track of versions that could not be recognized as semantic
            non_semantic_versions.append({"version": release, "upload_time": upload_time})

    return {
        "major_updates": sorted_versions(major_updates),
        "minor_updates": sorted_versions(minor_updates),
        "patch_updates": sorted_versions(patch_updates),
        "pre_release_updates": sorted_versions(pre_release_updates),
        "non_semantic_versions": non_semantic_versions,
    }


def get_parsed_environment_package_list():
    """
    Get a parsed list of packages in the current environment

    :return:
    """
    return parse_requirements_list(get_environment_requirements_list())


def get_environment_requirements_list():
    """
    Take the requirements list from the current running environment

    :return: string
    """
    requirement_list = []
    requirements = check_output([sys.executable, "-m", "pip", "freeze"])

    for requirement in requirements.split():
        requirement_list.append(requirement.decode("utf-8"))

    return requirement_list


def parse_requirements_list(requirements_list):
    """
    Take a list and return a list of dicts with {package, versions) based on the requirements specs

    :param requirements_list: string
    :return: string
    """
    req_list = []

    for requirement in requirements_list:
        requirement_no_comments = requirement.split("#")[0].strip()

        # if matching requirement line (Thing==1.2.3), update dict, continue
        req_match = re.match(
            r"\s*(?P<package>[^\s\[\]]+)(?P<extras>\[\S+\])?==(?P<version>\S+)",
            requirement_no_comments,
        )
        if req_match:
            req_list.append(
                {
                    "package": req_match.group("package"),
                    "version": req_match.group("version"),
                },
            )

    return req_list


async def get_pypi_package_data(package_name, version=None):
    """
    Get package data from pypi by the package name

    https://wiki.python.org/moin/PyPIJSON

    :param package_name: string
    :param version: string
    :return: dict
    """
    pypi_url = "https://pypi.org/pypi"

    if version:
        package_url = f"{pypi_url}/{package_name}/{version}/json"
    else:
        package_url = f"{pypi_url}/{package_name}/json"

    async with httpx.AsyncClient() as client:
        try:
            resp = await client.get(package_url, follow_redirects=True, timeout=None)
        except httpx.ConnectError:
            raise RuntimeError("Connection error!")

        # Package not available on pypi
        if resp.is_error:
            return None

        return resp.json()


async def get_package_update_list(package_name, version):
    """
    Return update information of a package from a given version

    :param package_name: string
    :param version: string
    :return: dict
    """
    package_version = semantic_version.Version.coerce(version)

    # Get package and version data from pypi
    package_data = await get_pypi_package_data(package_name)
    version_data = await get_pypi_package_data(package_name, version)

    # Current release specific information
    current_release = ""
    current_release_license = ""

    # Latest release specific information
    latest_release = ""
    latest_release_license = ""

    # Information about packages
    newer_releases = 0
    pre_releases = 0
    categorized_package_data = {
        "major_updates": [],
        "minor_updates": [],
        "patch_updates": [],
        "pre_release_updates": [],
        "non_semantic_versions": [],
    }

    if package_data:
        latest_release = package_data["info"]["version"]
        latest_release_license = package_data["info"]["license"] if package_data["info"]["license"] else ""
        categorized_package_data = get_categorized_package_data(package_data, package_version)

        # Get number of newer releases available for the given package, excluding pre_releases and non semantic versions
        newer_releases = len(
            categorized_package_data["major_updates"]
            + categorized_package_data["minor_updates"]
            + categorized_package_data["patch_updates"],
        )
        pre_releases = len(categorized_package_data["pre_release_updates"])

    if version_data:
        current_release = version_data["info"]["version"]
        current_release_license = version_data["info"]["license"] if version_data["info"]["license"] else ""

    return {
        "current_release": current_release,
        "current_release_license": current_release_license,
        "latest_release": latest_release,
        "latest_release_license": latest_release_license,
        "newer_releases": newer_releases,
        "pre_releases": pre_releases,
        **categorized_package_data,
    }



#!/usr/bin/env python
import asyncio
import json
import os
import unittest

from updatable import utils as updatable_utils

PATH = os.path.dirname(os.path.realpath(__file__))


async def get_pypi_package_data_monkey(package_name, version=None):
    if version:
        json_file = f"pypi-{package_name}-{version}.json"
    else:
        json_file = f"pypi-{package_name}.json"

    with open(os.path.join(PATH, "fixtures", json_file)) as data_file:
        return json.load(data_file)


class TestUpdateLicense(unittest.TestCase):
    """
    Tests package updatability
    """

    def setUp(self):
        self.get_pypi_package_data_orig = updatable_utils.get_pypi_package_data
        updatable_utils.get_pypi_package_data = get_pypi_package_data_monkey

    def tearDown(self):
        updatable_utils.get_pypi_package_data = self.get_pypi_package_data_orig

    def test_update_license(self):
        """
        Test update count for a package that has only major releases
        """
        updates = asyncio.run(updatable_utils.get_package_update_list("package3", "1.0.0"))
        self.assertEqual(updates["current_release"], "1.0.0")
        self.assertEqual(updates["latest_release"], "3.0.0")
        self.assertEqual(updates["current_release_license"], "GPL-2.0")
        self.assertEqual(updates["latest_release_license"], "MIT")

        updates = asyncio.run(updatable_utils.get_package_update_list("package3", "2.0.0"))
        self.assertEqual(updates["current_release"], "2.0.0")
        self.assertEqual(updates["latest_release"], "3.0.0")
        self.assertEqual(updates["current_release_license"], "GPL-3.0")
        self.assertEqual(updates["latest_release_license"], "MIT")

        updates = asyncio.run(updatable_utils.get_package_update_list("package3", "3.0.0"))
        self.assertEqual(updates["current_release"], "3.0.0")
        self.assertEqual(updates["latest_release"], "3.0.0")
        self.assertEqual(updates["current_release_license"], "MIT")
        self.assertEqual(updates["latest_release_license"], "MIT")


if __name__ == "__main__":
    unittest.main()



from updatable.utils import (
    get_categorized_package_data,
    get_environment_requirements_list,
    get_package_update_list,
    get_parsed_environment_package_list,
    get_pypi_package_data,
    is_major_update,
    is_minor_update,
    is_patch_update,
    parse_requirements_list,
    sorted_versions,
)

__all__ = [
    "is_major_update",
    "is_minor_update",
    "is_patch_update",
    "sorted_versions",
    "get_categorized_package_data",
    "get_parsed_environment_package_list",
    "get_environment_requirements_list",
    "parse_requirements_list",
    "get_pypi_package_data",
    "get_package_update_list",
]



#!/usr/bin/env python
import unittest

from tests.utils import get_environment_requirements_list_monkey
from updatable import utils as updatable_utils


class TestParse(unittest.TestCase):
    def setUp(self):
        self.get_environment_requirements_list_orig = updatable_utils.get_environment_requirements_list
        updatable_utils.get_environment_requirements_list = get_environment_requirements_list_monkey

    def tearDown(self):
        updatable_utils.get_environment_requirements_list = self.get_environment_requirements_list_orig

    def assert_package_list(self, packages):
        """
        Test if the requirements are parsed correctly

        :param packages: dict
        """
        self.assertEqual(len(packages), 5)

        self.assertEqual(packages[0]["version"], "0.1")
        self.assertEqual(packages[1]["version"], "1.0")
        self.assertEqual(packages[2]["version"], "2")
        self.assertEqual(packages[3]["version"], "2.4")
        self.assertEqual(packages[4]["version"], "3.0.0")

        self.assertEqual(packages[0]["package"], "package1")
        self.assertEqual(packages[1]["package"], "package2")
        self.assertEqual(packages[2]["package"], "package3")
        self.assertEqual(packages[3]["package"], "package4")
        self.assertEqual(packages[4]["package"], "package5")

    def test_get_parsed_environment_package_list(self):
        """
        Test parsing requirements from environment
        """
        packages = updatable_utils.get_parsed_environment_package_list()
        self.assert_package_list(packages)

    def test_parse_requirements_list(self):
        """
        Test parsing requirements list
        """
        requirements_list = [
            "package1==0.1",
            "package2==1.0",
            "package3==2",
            "package4==2.4",
            "package5==3.0.0",
        ]
        packages = updatable_utils.parse_requirements_list(requirements_list)
        self.assert_package_list(packages)


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
import asyncio
import datetime
import json
import os
import unittest
from unittest.mock import patch

import httpx
import respx

from updatable import utils as updatable_utils

PATH = os.path.dirname(os.path.realpath(__file__))


async def get_pypi_package_data_monkey(package_name, version=None):
    json_file = f"pypi-{package_name}.json"

    with open(os.path.join(PATH, "fixtures", json_file)) as data_file:
        return json.load(data_file)


class TestUpdate(unittest.TestCase):
    """
    Tests package updatability
    """

    def setUp(self):
        self.get_pypi_package_data_orig = updatable_utils.get_pypi_package_data
        updatable_utils.get_pypi_package_data = get_pypi_package_data_monkey

    def tearDown(self):
        updatable_utils.get_pypi_package_data = self.get_pypi_package_data_orig

    def test_major_update_count(self):
        """
        Test update count for a package that has only major releases
        """
        updates = asyncio.run(updatable_utils.get_package_update_list("package3", "1.0.0"))
        self.assertEqual(updates["newer_releases"], 2)
        self.assertEqual(len(updates["major_updates"]), 2)
        self.assertEqual(len(updates["minor_updates"]), 0)
        self.assertEqual(len(updates["patch_updates"]), 0)

        updates = asyncio.run(updatable_utils.get_package_update_list("package3", "2.0.0"))
        self.assertEqual(updates["newer_releases"], 1)
        self.assertEqual(len(updates["major_updates"]), 1)
        self.assertEqual(len(updates["minor_updates"]), 0)
        self.assertEqual(len(updates["patch_updates"]), 0)

        updates = asyncio.run(updatable_utils.get_package_update_list("package3", "3.0.0"))
        self.assertEqual(updates["newer_releases"], 0)
        self.assertEqual(len(updates["major_updates"]), 0)
        self.assertEqual(len(updates["minor_updates"]), 0)
        self.assertEqual(len(updates["patch_updates"]), 0)

    def test_major_update_version(self):
        """
        Test update versions for a package that has only major releases.
        This also shows that releases are ordered descendant by release date.
        """
        updates = asyncio.run(updatable_utils.get_package_update_list("package3", "1.0.0"))
        self.assertEqual(len(updates["major_updates"]), 2)
        self.assertEqual(updates["major_updates"][0]["version"], "3.0.0")
        self.assertEqual(updates["major_updates"][1]["version"], "2.0.0")

        updates = asyncio.run(updatable_utils.get_package_update_list("package3", "2.0.0"))
        self.assertEqual(len(updates["major_updates"]), 1)
        self.assertEqual(updates["major_updates"][0]["version"], "3.0.0")

        updates = asyncio.run(updatable_utils.get_package_update_list("package3", "3.0.0"))
        self.assertEqual(len(updates["major_updates"]), 0)

    def test_major_update_date(self):
        """
        Test update upload time for a package that has only major releases.
        This also shows that releases are ordered descendant by release date.
        """
        updates = asyncio.run(updatable_utils.get_package_update_list("package3", "1.0.0"))
        self.assertEqual(len(updates["major_updates"]), 2)
        self.assertEqual(
            updates["major_updates"][0]["upload_time"],
            datetime.datetime(year=2015, month=9, day=29, hour=23, minute=34, second=21),
        )
        self.assertEqual(
            updates["major_updates"][1]["upload_time"],
            datetime.datetime(year=2013, month=11, day=29, hour=23, minute=34, second=21),
        )

        updates = asyncio.run(updatable_utils.get_package_update_list("package3", "2.0.0"))
        self.assertEqual(len(updates["major_updates"]), 1)
        self.assertEqual(
            updates["major_updates"][0]["upload_time"],
            datetime.datetime(year=2015, month=9, day=29, hour=23, minute=34, second=21),
        )

        updates = asyncio.run(updatable_utils.get_package_update_list("package3", "3.0.0"))
        self.assertEqual(len(updates["major_updates"]), 0)

    def test_minor_update_count(self):
        """
        Test update count for a package that has only minor releases
        """
        updates = asyncio.run(updatable_utils.get_package_update_list("package2", "1.0.0"))
        self.assertEqual(updates["newer_releases"], 3)
        self.assertEqual(len(updates["major_updates"]), 0)
        self.assertEqual(len(updates["minor_updates"]), 3)
        self.assertEqual(len(updates["patch_updates"]), 0)

        updates = asyncio.run(updatable_utils.get_package_update_list("package2", "1.1.0"))
        self.assertEqual(updates["newer_releases"], 2)
        self.assertEqual(len(updates["major_updates"]), 0)
        self.assertEqual(len(updates["minor_updates"]), 2)
        self.assertEqual(len(updates["patch_updates"]), 0)

        updates = asyncio.run(updatable_utils.get_package_update_list("package2", "1.2.0"))
        self.assertEqual(updates["newer_releases"], 1)
        self.assertEqual(len(updates["major_updates"]), 0)
        self.assertEqual(len(updates["minor_updates"]), 1)
        self.assertEqual(len(updates["patch_updates"]), 0)

        updates = asyncio.run(updatable_utils.get_package_update_list("package2", "1.3.0"))
        self.assertEqual(updates["newer_releases"], 0)
        self.assertEqual(len(updates["major_updates"]), 0)
        self.assertEqual(len(updates["minor_updates"]), 0)
        self.assertEqual(len(updates["patch_updates"]), 0)

    def test_minor_update_version(self):
        """
        Test update versions for a package that has only minor releases.
        This also shows that releases are ordered descendant by release date.
        """
        updates = asyncio.run(updatable_utils.get_package_update_list("package2", "1.0.0"))
        self.assertEqual(len(updates["minor_updates"]), 3)
        self.assertEqual(updates["minor_updates"][0]["version"], "1.3.0")
        self.assertEqual(updates["minor_updates"][1]["version"], "1.2.0")
        self.assertEqual(updates["minor_updates"][2]["version"], "1.1.0")

        updates = asyncio.run(updatable_utils.get_package_update_list("package2", "1.1.0"))
        self.assertEqual(len(updates["minor_updates"]), 2)
        self.assertEqual(updates["minor_updates"][0]["version"], "1.3.0")
        self.assertEqual(updates["minor_updates"][1]["version"], "1.2.0")

        updates = asyncio.run(updatable_utils.get_package_update_list("package2", "1.2.0"))
        self.assertEqual(len(updates["minor_updates"]), 1)
        self.assertEqual(updates["minor_updates"][0]["version"], "1.3.0")

        updates = asyncio.run(updatable_utils.get_package_update_list("package2", "1.3.0"))
        self.assertEqual(len(updates["minor_updates"]), 0)

    def test_minor_update_date(self):
        """
        Test update upload time for a package that has only minor releases.
        This also shows that releases are ordered descendant by release date.
        """
        updates = asyncio.run(updatable_utils.get_package_update_list("package2", "1.0.0"))
        self.assertEqual(len(updates["minor_updates"]), 3)
        self.assertEqual(
            updates["minor_updates"][0]["upload_time"],
            datetime.datetime(year=2015, month=9, day=29, hour=23, minute=34, second=21),
        )
        self.assertEqual(
            updates["minor_updates"][1]["upload_time"],
            datetime.datetime(year=2014, month=9, day=29, hour=23, minute=34, second=21),
        )
        self.assertEqual(
            updates["minor_updates"][2]["upload_time"],
            datetime.datetime(year=2013, month=10, day=29, hour=23, minute=34, second=21),
        )

        updates = asyncio.run(updatable_utils.get_package_update_list("package2", "1.1.0"))
        self.assertEqual(len(updates["minor_updates"]), 2)
        self.assertEqual(
            updates["minor_updates"][0]["upload_time"],
            datetime.datetime(year=2015, month=9, day=29, hour=23, minute=34, second=21),
        )
        self.assertEqual(
            updates["minor_updates"][1]["upload_time"],
            datetime.datetime(year=2014, month=9, day=29, hour=23, minute=34, second=21),
        )

        updates = asyncio.run(updatable_utils.get_package_update_list("package2", "1.2.0"))
        self.assertEqual(len(updates["minor_updates"]), 1)
        self.assertEqual(
            updates["minor_updates"][0]["upload_time"],
            datetime.datetime(year=2015, month=9, day=29, hour=23, minute=34, second=21),
        )

        updates = asyncio.run(updatable_utils.get_package_update_list("package2", "1.3.0"))
        self.assertEqual(len(updates["minor_updates"]), 0)

    def test_patch_update_count(self):
        """
        Test update count for a package that has only patch releases
        """
        updates = asyncio.run(updatable_utils.get_package_update_list("package1", "1.0.0"))
        self.assertEqual(updates["newer_releases"], 2)
        self.assertEqual(len(updates["major_updates"]), 0)
        self.assertEqual(len(updates["minor_updates"]), 0)
        self.assertEqual(len(updates["patch_updates"]), 2)

        updates = asyncio.run(updatable_utils.get_package_update_list("package1", "1.0.1"))
        self.assertEqual(updates["newer_releases"], 1)
        self.assertEqual(len(updates["major_updates"]), 0)
        self.assertEqual(len(updates["minor_updates"]), 0)
        self.assertEqual(len(updates["patch_updates"]), 1)

        updates = asyncio.run(updatable_utils.get_package_update_list("package1", "1.0.2"))
        self.assertEqual(updates["newer_releases"], 0)
        self.assertEqual(len(updates["major_updates"]), 0)
        self.assertEqual(len(updates["minor_updates"]), 0)
        self.assertEqual(len(updates["patch_updates"]), 0)

    def test_patch_update_version(self):
        """
        Test update versions for a package that has only patch releases.
        This also shows that releases are ordered descendant by release date.
        """
        updates = asyncio.run(updatable_utils.get_package_update_list("package1", "1.0.0"))
        self.assertEqual(len(updates["patch_updates"]), 2)
        self.assertEqual(updates["patch_updates"][0]["version"], "1.0.2")
        self.assertEqual(updates["patch_updates"][1]["version"], "1.0.1")

        updates = asyncio.run(updatable_utils.get_package_update_list("package1", "1.0.1"))
        self.assertEqual(len(updates["patch_updates"]), 1)
        self.assertEqual(updates["patch_updates"][0]["version"], "1.0.2")

        updates = asyncio.run(updatable_utils.get_package_update_list("package1", "1.0.2"))
        self.assertEqual(len(updates["patch_updates"]), 0)

    def test_patch_update_date(self):
        """
        Test update upload time for a package that has only patch releases.
        This also shows that releases are ordered descendant by release date.
        """
        updates = asyncio.run(updatable_utils.get_package_update_list("package1", "1.0.0"))
        self.assertEqual(len(updates["patch_updates"]), 2)
        self.assertEqual(
            updates["patch_updates"][0]["upload_time"],
            datetime.datetime(year=2013, month=10, day=22, hour=23, minute=34, second=21),
        )
        self.assertEqual(
            updates["patch_updates"][1]["upload_time"],
            datetime.datetime(year=2013, month=9, day=29, hour=23, minute=34, second=21),
        )

        updates = asyncio.run(updatable_utils.get_package_update_list("package1", "1.0.1"))
        self.assertEqual(len(updates["patch_updates"]), 1)
        self.assertEqual(
            updates["patch_updates"][0]["upload_time"],
            datetime.datetime(year=2013, month=10, day=22, hour=23, minute=34, second=21),
        )

        updates = asyncio.run(updatable_utils.get_package_update_list("package1", "1.0.2"))
        self.assertEqual(len(updates["patch_updates"]), 0)

    def test_unusual_semantic_count(self):
        """
        Test for packages for a package with unusual versions
        """
        updates = asyncio.run(updatable_utils.get_package_update_list("package5", "0.9999999"))
        self.assertEqual(updates["newer_releases"], 2)
        self.assertEqual(len(updates["major_updates"]), 0)
        self.assertEqual(len(updates["minor_updates"]), 2)
        self.assertEqual(len(updates["patch_updates"]), 0)

        updates = asyncio.run(updatable_utils.get_package_update_list("package5", "0.99999999"))
        self.assertEqual(updates["newer_releases"], 1)
        self.assertEqual(len(updates["major_updates"]), 0)
        self.assertEqual(len(updates["minor_updates"]), 1)
        self.assertEqual(len(updates["patch_updates"]), 0)

        updates = asyncio.run(updatable_utils.get_package_update_list("package5", "0.999999999"))
        self.assertEqual(updates["newer_releases"], 0)
        self.assertEqual(len(updates["major_updates"]), 0)
        self.assertEqual(len(updates["minor_updates"]), 0)
        self.assertEqual(len(updates["patch_updates"]), 0)

    def test_non_semantic_count(self):
        """
        Test for package with non semantic verions
        """
        updates = asyncio.run(updatable_utils.get_package_update_list("package6", "0.1"))
        self.assertEqual(updates["newer_releases"], 0)
        self.assertEqual(len(updates["major_updates"]), 0)
        self.assertEqual(len(updates["minor_updates"]), 0)
        self.assertEqual(len(updates["patch_updates"]), 0)
        self.assertEqual(len(updates["non_semantic_versions"]), 3)


class TestGetPackageData(unittest.TestCase):
    def setUp(self) -> None:
        respx.get("https://pypi.org/pypi/updatable/json").respond(status_code=200, json={"test1": "ok"})
        respx.get("https://pypi.org/pypi/updatable/1.0.0/json").respond(status_code=200, json={"test2": "ok"})
        respx.get("https://pypi.org/pypi/updatable/2.0.0/json").respond(status_code=404, json={"message": "Not Found"})
        respx.get("https://pypi.org/pypi/updatable/2.5.0/json").mock(side_effect=httpx.ConnectError)

    @respx.mock
    def test_get_pypi_package_data_no_version(self):
        """
        Assures that fetched pypi data is parsed correctly if no version is given
        """
        response = asyncio.run(updatable_utils.get_pypi_package_data("updatable"))
        self.assertDictEqual(response, {"test1": "ok"})

    @respx.mock
    def test_get_pypi_package_data_existing_version(self):
        """
        Assures that fetched pypi data is parsed correctly if a valid version is given
        """
        response = asyncio.run(updatable_utils.get_pypi_package_data("updatable", "1.0.0"))
        self.assertDictEqual(response, {"test2": "ok"})

    @respx.mock
    def test_get_pypi_package_data_with_non_existing_version(self):
        """
        Assures that `None is return if an invalid version is given
        """
        response = asyncio.run(updatable_utils.get_pypi_package_data("updatable", "2.0.0"))
        self.assertEqual(response, None)

    @respx.mock
    def test_get_pypi_package_data_with_connection_error(self):
        """
        Assures a RuntimeError is raised on connection error
        """
        with self.assertRaises(RuntimeError):
            asyncio.run(updatable_utils.get_pypi_package_data("updatable", "2.5.0"))


class TestGetEnvironmentList(unittest.TestCase):
    def _mocked_subprocess_check_output(*args, **kwargs):
        """
        This method is used to mock the check_output function from subprocess,
        which captures the output on the pip freeze command in `get_environment_requirements_list`
        """
        return b"""
        package1==1.0.0
        package2==1.2.1
        package3==2.5.3
        """

    def test_get_mocked_environment_requirements_list(self):
        """
        Assures that list of mocked requirenments can be loaded from the cureent environemnt correctly
        """

        with patch(
            "updatable.utils.check_output",
            side_effect=self._mocked_subprocess_check_output,
        ) as mock:
            package_list = updatable_utils.get_environment_requirements_list()
            self.assertTrue(mock.called)
            self.assertListEqual(
                package_list,
                [
                    "package1==1.0.0",
                    "package2==1.2.1",
                    "package3==2.5.3",
                ],
            )

    def test_get_environment_requirements_list(self):
        """
        Assures that list of requirenments can be loaded from the cureent environemnt correctly
        """
        package_list = updatable_utils.get_environment_requirements_list()
        self.assertTrue(len(package_list) > 0)


if __name__ == "__main__":
    unittest.main()
